﻿using System.Net.Http.Headers;
using Newtonsoft.Json;
using OpenApiConsoleExample.OpenApiClients;

// Get an access token from Access Management
const string tridionDocsUrl = "https://tridiondocs.example.com";
const string serviceAccountClientId = "TridionDocs_CM_API_ClientID";
const string serviceAccountClientSecret = "TridionDocs_CM_API_ClientSecret";

AccessToken accessToken = await GetAccessToken(tridionDocsUrl, serviceAccountClientId, serviceAccountClientSecret);

// Setup the authenticated communication to Tridion Docs
HttpClient httpClient= new()
{
    BaseAddress = new Uri(tridionDocsUrl + "/ISHWS/Api/")
};
httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue(accessToken.TokenType, accessToken.Token);
TridionDocsClient tridionDocsClient = new(httpClient);

// Make a call to ask the Tridion Docs version
Console.WriteLine($"Tridion Docs version: {await tridionDocsClient.GetApplicationVersionAsync()}");

// Aks Tridion Docs who I am
User user = await tridionDocsClient.GetCurrentUserAsync(SelectedProperties.TitleAndDescription, FieldGroup.None, null, false);
Console.WriteLine($"I am {user.Title} with identifier {user.Id}");

return;

// Get an access token from Access Management using client credential flow
async Task<AccessToken> GetAccessToken(string authenticationServerUrl, string clientId, string clientSecret)
{
    HttpClient client = new()
    {
        BaseAddress = new Uri(authenticationServerUrl)            
    };

    List<KeyValuePair<string, string>> body =
    [
        new KeyValuePair<string, string>("grant_type", "client_credentials"),
        new KeyValuePair<string, string>("client_id", clientId),
        new KeyValuePair<string, string>("client_secret", clientSecret)
    ];

    using HttpResponseMessage responseMessage = await client.PostAsync("/ISHAM/connect/token", new FormUrlEncodedContent(body));

    string token = await responseMessage.Content.ReadAsStringAsync();

    AccessToken serviceAccountToken = JsonConvert.DeserializeObject<AccessToken>(token)!;

    return serviceAccountToken;
}

internal record AccessToken(
    [property:JsonProperty("access_token")]
    string Token,

    [property:JsonProperty("expires_in")]
    int ExpiresIn,

    [property:JsonProperty("token_type")]
    string TokenType
);